local BH = ByeHakmud 
BH.gui = {}
local gui = BH.gui
local vars = BH.Vars
 
 --- Config Panel View Accessors
 
function gui:GetConfigFrame()
    return self.configFrame
end
 
function gui:GetTabUsedIn()
    return self.tabUsedIn
end

function gui:SetTabUsedIn(frame)
    self.tabUsedIn = frame
end
 
function gui:GetTabMainSettings()
    return self.tabMainSettings;
end

function gui:SetTabMainSettings(frame)
    self.tabMainSettings = frame
end

function gui:GetMainSettingsWidgetByName(name)
    local panel = gui:GetTabMainSettings();
    return _G[panel:GetName()..name];
end
 
function gui:GetCBOnlyEjectWhenInParty()
    return gui:GetMainSettingsWidgetByName("_CB_OnlyEjectWhenInParty");
end

function gui:GetBHChannelToNotifyCB()
    return gui:GetMainSettingsWidgetByName("_CB_CHANNEL_TO_NOTIFY");
end

function gui:GetBHNotifyMsgEB()
    return gui:GetMainSettingsWidgetByName("_EB_NOTIFY_MSG");
end

--- Config Panel View Initialization
function gui:GetConfigPanelName()
    return "Bye Hakmud"
end

function gui:AddConfigPanelToAddonOptions()
	local panel = self:GetConfigFrame();
	panel.name = self:GetConfigPanelName()
	panel.okay = function (self) BH_OnOk(); end;
	panel.refresh = function (self) self:TabMainSettingsUpdate(); end;
	InterfaceOptions_AddCategory(panel);
end

--- Config Panel Controller

function gui:CB_OnlyEjectWhenInPartyOnClick(checkButton)
    local enabled = checkButton:GetChecked() == 1;
    vars:SetOnlyEjectWhenInParty(enabled);
end

function gui:TabMainSettingsUpdate()
	gui:GetCBOnlyEjectWhenInParty():SetChecked(vars:IsOnlyEjectWhenInParty());
	gui:RB_Model_SetKeptOnShift(BH.Vars:IsHakmudKeptOnShift());
	local channelToNotify = vars:GetChannelToNotify()
	gui:CB_ChannelToNotifyOnLoad(gui:GetBHChannelToNotifyCB())
	gui:GetBHNotifyMsgEB():SetText(vars:GetNotifyMsg())
	local creatureID = UIDropDownMenu_GetSelectedValue(gui.cbMountSelector);
	gui:UpdateSeatCheckBoxes(creatureID)
end

function gui:ConfigFrame_OnLoad(frame)
	BH:SetEventFrame(frame)
	self.configFrame = frame
	gui:AddConfigPanelToAddonOptions();

	local subFrame = _G[frame:GetName().."SubText"]
	if subFrame then 
	    subFrame:SetText(TL(
		"ByeHakmud can automatically dismount a vendor when "..
		"a mount like the Traveler's Tundra Mammoth is invoked. "..
		"That can be useful if you want to keep a seat free for a "..
		"group member to join your mount and repair his armor while riding."
	      ));
	end

	subFrame = _G[frame:GetName().."VersionText"]
	if subFrame then
	    subFrame:SetText(BH.Vars:GetAddonUrl().."\n"..BH.Vars:GetVersion());
	end

	frame:RegisterEvent("ADDON_LOADED");
end

--- Controller of Radio Button 'Hakmud kept on <shift>'
function gui:RB_Model_IsKeptOnShift()
	return gui.RB_Model_KeptOnShift;
end

function gui:RB_Model_SetKeptOnShift(keep)
	gui.RB_Model_KeptOnShift = keep;
	vars:SetHakmudKeptOnShift(keep)
	gui:UpdateHakmudKeptOnShiftButtons();
end

function gui:UpdateHakmudKeptOnShiftButtons()
	local keptOnShift = gui:RB_Model_IsKeptOnShift();
	gui:GetMainSettingsWidgetByName("_RB_SHIFT_KEEPS"):SetChecked(keptOnShift);
	gui:GetMainSettingsWidgetByName("_RB_SHIFT_KICKS"):SetChecked(not keptOnShift);
end

---
function BH_DropDownCallback(info)
    UIDropDownMenu_SetSelectedValue(info.owner, info.value)
end

local entry = {}
function gui:CB_AddEntry(owner, text, value, tooltipText, func)
    entry.owner	= owner
    entry.text	= text
    entry.value	= value
    entry.checked = false
    entry.func = func
    
    if tooltipText then
	entry.tooltipText = tostring(tooltipText)
	entry.tooltipTitle = tostring(text)
	entry.tooltipOnButton = 1
	entry.tooltipWhileDisabled = 1
    else
	entry.tooltipText = nil
	entry.tooltipTitle = nil
	entry.tooltipOnButton = nil
	entry.tooltipWhileDisabled = nil
    end
    
    UIDropDownMenu_AddButton(entry)
end

function BH_CB_ChannelToNotifyCallback(info)
    BH_DropDownCallback(info)
    vars:SetChannelToNotify(info.value)
end

function BH_CB_ChannelToNotifyInitialize(frame)
    gui:CB_AddEntry(frame, TL("don't post"), "none", TL("Don't post anything"), BH_CB_ChannelToNotifyCallback)
    gui:CB_AddEntry(frame, TL("local"), "local", TL("Show a message locally. No other players will be bothered with this message."), BH_CB_ChannelToNotifyCallback)
    gui:CB_AddEntry(frame, TL("say"), "say", TL("Say the message (/s). Nearby players will receive the message wether in your group or not."), BH_CB_ChannelToNotifyCallback)
    gui:CB_AddEntry(frame, TL("auto"), "auto", TL("Automatically tries to determine wether you are in an instance, party, raid or on a battleground and posts the message to that channel."), BH_CB_ChannelToNotifyCallback)
    gui:CB_AddEntry(frame, TL("auto / say"), "auto say", TL("Just like auto, but use 'say' when not in a group."), BH_CB_ChannelToNotifyCallback)
    gui:CB_AddEntry(frame, TL("instance only"), "instance", TL("Posts the message in the instance channel (/i)"), BH_CB_ChannelToNotifyCallback)
    gui:CB_AddEntry(frame, TL("party only"), "party", TL("Posts the message in the party channel (/p)"), BH_CB_ChannelToNotifyCallback)
    gui:CB_AddEntry(frame, TL("raid only"), "raid", TL("Posts the message in the raid channel (/ra)"), BH_CB_ChannelToNotifyCallback)
    gui:CB_AddEntry(frame, TL("battleground only"), "battleground", TL("Posts the message in the battleground channel (/bg)"), BH_CB_ChannelToNotifyCallback)
    gui:CB_AddEntry(frame, TL("yell"), "yell", TL("Yell the message (/y)"), BH_CB_ChannelToNotifyCallback)
end

function gui:CB_ChannelToNotifyOnLoad(frame)
    UIDropDownMenu_Initialize(frame, BH_CB_ChannelToNotifyInitialize)
    local selVal = BH.Vars:GetChannelToNotify()
    UIDropDownMenu_SetSelectedValue(frame, selVal)
end

--- Notify Msg Edit Box
function gui:EB_NotifyMSGOnShow(editBox)
    editBox:SetText("")
    editBox:SetText(BH.Vars:GetNotifyMsg())
end

function gui:EBNotifyMsgOnEnterPressed(editBox)
    local notifyMsg = editBox:GetText()
    vars:SetNotifyMsg(notifyMsg)
end
--- Mount Selector
 
function gui:ApplySeatCheckBoxes()
    local mountSelector = gui:GetMainSettingsWidgetByName("_CB_MountSelector");
    local creatureID = UIDropDownMenu_GetSelectedValue(mountSelector);
    if creatureID then
	local seatsToEject = {}	    
	local seatsToEjectIdx = 1
	local checkBoxID = 1
	local checkBox = nul
	repeat
	    local checkBox = gui:GetMainSettingsWidgetByName("_CB_EjectVendor"..checkBoxID)
	    if checkBox then
		local checked = checkBox:GetChecked()
		if checked then
		    seatsToEject[seatsToEjectIdx] = checkBoxID
		    seatsToEjectIdx = seatsToEjectIdx + 1
		end
		checkBoxID = checkBoxID + 1
	    end
	until checkBox == nil
	BH:SetSeatsToEject4Mount(creatureID, seatsToEject)
    end
end

function gui:UpdateSeatCheckBoxes(creatureID)
    local seatsToEject = BH:GetSeatsToEject4Mount(creatureID);
    local checkBoxID = 1
    local checkBox = nul
    repeat
	local checkBox = gui:GetMainSettingsWidgetByName("_CB_EjectVendor"..checkBoxID)
	if checkBox then
	    local checked = false
	    if seatsToEject then
	      for i, seatID in ipairs(seatsToEject) do
		  if seatID == checkBoxID then
		      checked = true
		  end
	      end
	    end
	    checkBox:SetChecked(checked)
	    checkBoxID = checkBoxID + 1
	end
    until checkBox == nil
end

function BH_CB_MountSelectorEntryCallback(info)
    gui:ApplySeatCheckBoxes()
    BH_DropDownCallback(info)
    gui:UpdateSeatCheckBoxes(info.value)
end

function BH_CB_MountSelectorInitialize(frame)
    local companionCount = GetNumCompanions("MOUNT");
    for i = 1, companionCount do
	local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i);
	local seatsToEject = BH:GetSeatsToEject4Mount(creatureID);
	if seatsToEject then
	    gui:CB_AddEntry(frame, creatureName, creatureID, nil, BH_CB_MountSelectorEntryCallback)
	end
    end
end

function gui:CB_MountSelectorOnLoad(frame)
    gui.cbMountSelector = frame
    gui:CB_MountSelectorOnShow(frame)
end

function gui:CB_MountSelectorOnShow(frame)
    UIDropDownMenu_Initialize(frame, BH_CB_MountSelectorInitialize)
    if BH.Vars:IsLoaded() then
	local selectedCreatureID = UIDropDownMenu_GetSelectedValue(frame);
	
	local companionCount = GetNumCompanions("MOUNT");
	for i = 1, companionCount do
	    local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i);
	    local seatsToEject = BH:GetSeatsToEject4Mount(creatureID);
	    if seatsToEject then
		if issummoned or selectedCreatureID == nil then
		    selectedCreatureID = creatureID
		end
	    end
	end
	UIDropDownMenu_SetSelectedValue(frame, selectedCreatureID)
	gui:UpdateSeatCheckBoxes(selectedCreatureID)
    end
end

function gui:CBEjectVendor(checkButton)
    gui:ApplySeatCheckBoxes()
end

function BH_BoardingPermissionsOnLoad(frame)
    local panelPermissions = frame
    panelPermissions.name = TL("Boarding permissions")
    _G[panelPermissions:GetName().."Title"]:SetText(panelPermissions.name)
    _G[panelPermissions:GetName().."RuleDescription"]:SetText(
	TL("When a group member tries to board your mount these rules are "..
	    "evaluated from top to down. The first rule that matches that "..
	    "group member determines his boarding permission."
	)
    )
end

local BH_RuleTooltipTexts = {
    ["target"] = {
	["Allow"]     = TL("Allows your fellow to join your mount when you have targeted him. If you have targeted something else / nothing this rule doesn't apply.");
	["Deny"]      = TL("Denies your fellow to join your mount when you have targeted him. If you have targeted something else / nothing this rule doesn't apply.");
	["DontCheck"] = TL("Disables this rule");
    };
    ["focus"] = {
	["Allow"]     = TL("Allows your fellow to join your mount when you have focussed him. If you have targeted something else / nothing this rule doesn't apply.");
	["Deny"]      = TL("Denies your fellow to join your mount when you have focussed him. If you have targeted something else / nothing this rule doesn't apply.");
	["DontCheck"] = TL("Disables this rule");
    };
    ["friend"] = {
	["Allow"]     = TL("Allows your fellow to join your mount if he is on your friend list. If he isn't listed as your friend this rule doesn't apply.");
	["Deny"]      = TL("Denies your fellow to join your mount if he is on your friend list. If he isn't listed as your friend this rule doesn't apply.");
	["DontCheck"] = TL("Disables this rule");
    };
    ["guild"] = {
	["Allow"]     = TL("Allows your fellow to join your mount if he is in your guild. If he isn't in your guild this rule doesn't apply.");
	["Deny"]      = TL("Denies your fellow to join your mount if he is in your guild. If he isn't in your guild this rule doesn't apply.");
	["DontCheck"] = TL("Disables this rule");
    };
    ["group"] = {
	["Allow"]     = TL("Allows your fellow to join your mount.");
	["Deny"]      = TL("Denies your fellow to join your mount.");
	["DontCheck"] = TL("Disables this rule");
    };    
    ["_default"] = {
	["Allow"]     = TL("Allows this player to join your mount.");
	["Deny"]      = TL("Denies this player to join your mount.");
	["DontCheck"] = TL("Disables this rule");
    };    
}

function BG_GetRuleTooltipText(ruleName, ruleType)
    local retVal
    types = BH_RuleTooltipTexts[ruleName]
    if types then
	retVal = types[ruleType]
    end
    
    if retVal == nil then
	retVal = BH_RuleTooltipTexts["_default"][ruleType]
    end
    return retVal
end

function gui:ScrollFrameOnLoad(scrollFrame)
    gui.scrollFrame = scrollFrame
end

function BH_ScrollFrameUpdateFunction(frame)
    local entryCount = 6
    local lineHeight = 18

    -- hide all lines and set their id to 0, the update function is responsible for showing and setting id's of valid lines	
    local entry = frame:GetParent():GetName().."Entry"
    for i = 1, entryCount do
	local entryBtn = _G[ entry..i ]
	if entryBtn then
	    entryBtn:SetID(0)
	    entryBtn:Hide()
	    _G[ entry..i.."NameNormalText" ].name = nil
	end
    end
    
    local offset = FauxScrollFrame_GetOffset(frame)
    local playerIdx = 1
    local playerPermissions = BH:GetPlayerPermissions()
    if playerPermissions then 
	for player, permission in BH.PairsByKeys(playerPermissions) do
	    local entryIdx = playerIdx - offset
	    if entryIdx > 0 and entryIdx <= entryCount then
		local entryName = entry..entryIdx
		if _G[ entryName.."NameNormalText" ] then
		    _G[ entryName.."NameNormalText" ].name = player
		    
		    local playerTL = player
		    local iconPrefix = "|TInterface\\FriendsFrame\\UI-Toast-FriendOnlineIcon:32|t"
		    for defPermIdx, defPerm in pairs(BH:GetDefaultPlayerPermissions()) do
			if player == defPerm then
			    playerTL = TL(player)
			    iconPrefix = "|TInterface\\Buttons\\LockButton-Locked-Up:32|t"
			    break
			end
		    end
		    
		    if permission == "Allow" then
			coloredName = iconPrefix.."|cFF00FF00"..playerTL.."|r"
		    elseif permission == "Deny" then
			coloredName = iconPrefix.."|cFFFF0000"..playerTL.."|r"
		    elseif permission == "DontCheck" then
			coloredName = iconPrefix.."|cFF7F7F7F"..playerTL.."|r"
		    end
		    
		    _G[ entryName.."NameNormalText" ]:SetText(coloredName)
		    
		    local cbTypes = {"Allow","Deny","DontCheck"}
		    for i, cbType in pairs(cbTypes) do
			local chbx = _G[ entryName..cbType ]
			chbx:SetChecked((permission == cbType) and 1 or nil)
			chbx.tooltipText = BG_GetRuleTooltipText(player, cbType)
		    end
		    
		    _G[ entryName.."DontCheck" ]:SetChecked((permission == "DontCheck") and 1 or nil)
		    _G[ entryName ]:SetID(playerIdx)
		    _G[ entryName ]:Show()
		end
	    end
	    
	    entryIdx = entryIdx + 1
	    playerIdx = playerIdx + 1
	end
    end
    
    local scrollFrame = _G[ frame:GetName()]
    local scrollNumItems = playerIdx
    local scrollNumToDisplay = entryCount
    local scrollValueStep = lineHeight
    
    FauxScrollFrame_Update( scrollFrame, scrollNumItems, scrollNumToDisplay, scrollValueStep);
end

function BH_PermissionCheckbox_OnClick(checkbox)
    local parentName = checkbox:GetParent():GetName()
    local playerName = _G[ parentName.."NameNormalText" ].name

    local prefixLen = string.len(parentName) + string.len(tostring(checkbox:GetID()))
    local permission = string.sub(checkbox:GetName(), prefixLen)

    BH:GetPlayerPermissions()[playerName] = permission
    
    if gui.scrollFrame then BH_ScrollFrameUpdateFunction(gui.scrollFrame) end
end

-- 
function gui.DelPresetOnClick(button)
    local presetName = UIDropDownMenu_GetSelectedValue(gui.preset);
    vars:SetEditorPresetName(nil)
    vars:DeletePreset(presetName)
    gui:PresetOnShow(gui.preset)
    gui:TabsUpdate()
end

function gui:AddPresetOnClick(button)
    local editBox = _G[button:GetParent():GetName() .."PresetName"]
    local presetName = editBox:GetText()
    vars:CreatePreset(presetName)
    vars:SetEditorPresetName(presetName)
    self:PresetOnShow(self.preset)
    if oldPresetName == nil then
      local btn = _G[self:GetConfigFrame():GetName().."Tab1"]
      self:TabOnClick(btn)
    else
      self:TabsUpdate()
    end
end

function gui:TabsUpdate(clickedButton)
    local configFrame = self:GetConfigFrame() 
    local presetName = vars:GetEditorPresetName()
    if configFrame and (clickedButton or (presetName == nil)) then
      for btnIdx = 1, configFrame.numTabs do
	local btnName = configFrame:GetName().."Tab"..btnIdx
	local btn = _G[btnName]
	
	if presetName and clickedButton and btn:GetID() == clickedButton:GetID() then
	  _G[btn.tabFrameName]:Show()
	else 
	  _G[btn.tabFrameName]:Hide()
	end
      end
    end
    
    if presetName ~= nil then
      self:TabMainSettingsUpdate()
      if self.scrollFrame then BH_ScrollFrameUpdateFunction(self.scrollFrame) end
      self:TabUsedInUpdate()
    end
end

--- preset selector
function gui.PresetOnSelect(info)
    local oldPresetName = vars:GetEditorPresetName()
    gui:EBNotifyMsgOnEnterPressed(gui:GetBHNotifyMsgEB())
    
    BH_DropDownCallback(info)
    
    vars:SetEditorPresetName(info.value)
    gui:TabsUpdate()
end

function gui.PresetInitialize(frame)
    local presets = vars:GetPresets()
    for presetName, preset in pairs(presets) do
	gui:CB_AddEntry(frame, presetName, presetName, nil, gui.PresetOnSelect)
    end
end

function gui:PresetOnLoad(frame)
    self.preset = frame
    self:PresetOnShow(frame)
end

function gui:PresetOnShow(frame)
    UIDropDownMenu_Initialize(frame, gui.PresetInitialize)
    local selVal = vars:GetEditorPresetName()
    local redrawNeeded = false
    if selVal == nil then
	selVal = next(vars:GetPresets())
	vars:SetEditorPresetName(selVal)
	redrawNeeded = true
    end
    UIDropDownMenu_SetSelectedValue(frame, selVal)
    
    if redrawNeeded then
	local configFrame = self:GetConfigFrame()
	if configFrame then
	  local btn = _G[configFrame:GetName().."Tab1"]
	  if btn then
	    self:TabOnClick(btn)
	  end
	end
    end
end
----

function gui:TabUsedInUpdate()
    local tabUsedIn = self:GetTabUsedIn()
    if tabUsedIn then
      local parentName = tabUsedIn:GetName()
      
      _G[parentName.."World"]:SetChecked(vars:IsUsedInWorld())
      _G[parentName.."Scenario"]:SetChecked(vars:IsUsedInScenario())
      _G[parentName.."Dungeon"]:SetChecked(vars:IsUsedInDungeon())
      _G[parentName.."Raid"]:SetChecked(vars:IsUsedInRaid())
      _G[parentName.."Arena"]:SetChecked(vars:IsUsedInArena())
      _G[parentName.."Battlegrounds"]:SetChecked(vars:IsUsedInBattlegrounds())
      _G[parentName.."Random"]:SetChecked(vars:IsUsedInRandomParty())
      _G[parentName.."Mixed"]:SetChecked(vars:IsUsedInMixedParty())
      _G[parentName.."Guild"]:SetChecked(vars:IsUsedInGuildParty())
      _G[parentName.."PvE"]:SetChecked(vars:IsUsedInPvE())
      _G[parentName.."PvP"]:SetChecked(vars:IsUsedInPvP())
      
      gui:PresetWarningUpdate()
    end
end

function gui:TabOnClick(clickedButton)
    PanelTemplates_Tab_OnClick(clickedButton, clickedButton:GetParent());
    gui:TabsUpdate(clickedButton)
end

gui.situation = {}
gui.instanceType2BtnLabel = {
  none = "World",
  scenario = "Scenario",
  party = "Dungeon",
  raid = "Raid",
  arena = "Arena",
  pvp = "Battleground",
}
function gui:CreatePresetConflictWarning()
    local warning
    local concernsEditorPreset = false
    local situation = self.situation
    for _, instanceType in pairs(BH:GetInstanceTypes()) do
	situation.instanceType = instanceType
	for _, groupType in pairs({"Random party", "Mixed party", "Guild party"}) do
	    if groupType == "Random party" then
		situation.numGuildPresent = 0
		situation.numGroupMembers = 5
	    elseif groupType == "Mixed party" then
		situation.numGuildPresent = 3
		situation.numGroupMembers = 5
	    else 
		situation.numGuildPresent = 5
		situation.numGroupMembers = 5
	    end
	
	    for _, isPvP in pairs({true, false}) do
		situation.isPvP = isPvP
		local pre4Sit = BH:GetPresets4Situation(situation)
		if #pre4Sit > 1 then
		    local pvpstr
		    if situation.isPvP then
		      pvpstr = TL("PvP")
		    else 
		      pvpstr = TL("PvE")
		    end
					      
		    local situationstr = "("..
		      TL(self.instanceType2BtnLabel[situation.instanceType]).. " + ".. 
		      TL(groupType).. " + ".. 
		      pvpstr.. ")"
		      
		    local presetNames
		    for _, presetName in pairs(pre4Sit) do
		      concernsEditorPreset = concernsEditorPreset or presetName == vars:GetEditorPresetName()
		      if presetNames then
			presetNames = presetNames .. " \"" .. presetName.."\""
		      else 
			presetNames = "\""..presetName.."\""
		      end
		    end
		    
  --			local warningTxt = "|TInterface\PVPFrame\Icon-Combat:16|t"..
		    warning = "!!! "..
		      TL("Conflict detected").." !!!\n\n"..
		      TL("in presets").." "..presetNames.."\n\n"..
		      TL("for settings").." "..situationstr
		    
		    if concernsEditorPreset then break end
		end
	    end
	    if concernsEditorPreset then break end
	end
	if concernsEditorPreset then break end
    end
    return warning
end

function gui:IsPresetUsable(presetName)
    local instanceOk = false
    instanceOk = instanceOk or vars:IsUsedInWorld(presetName)
    instanceOk = instanceOk or vars:IsUsedInScenario(presetName)
    instanceOk = instanceOk or vars:IsUsedInDungeon(presetName)
    instanceOk = instanceOk or vars:IsUsedInRaid(presetName)
    instanceOk = instanceOk or vars:IsUsedInArena(presetName)
    instanceOk = instanceOk or vars:IsUsedInBattlegrounds(presetName)
    
    local partyOk = false
    partyOk = partyOk or vars:IsUsedInRandomParty(presetName)
    partyOk = partyOk or vars:IsUsedInMixedParty(presetName)
    partyOk = partyOk or vars:IsUsedInGuildParty(presetName)
    
    local pvpOk = false
    pvpOk = pvpOk or vars:IsUsedInPvE(presetName)
    pvpOk = pvpOk or vars:IsUsedInPvP(presetName)
    
    return instanceOk and partyOk and pvpOk
end

function gui:PresetWarningUpdate()
    local tabUsedIn = self:GetTabUsedIn()
    if tabUsedIn then
	local ctrlWarning = _G[self:GetTabUsedIn():GetName().."Warning"]
	local warning = self:CreatePresetConflictWarning()
	if warning == nil then
	    if not self:IsPresetUsable(vars:GetEditorPresetName()) then
		warning = TL("This preset is inactive because you specified no circumstances under which it could be used.")
	    end
	end
	ctrlWarning:SetText(warning)
    end
end

-- Debug Code
function BH_ShowFrameProps(frame)
    print(tostring(tostring(frame:GetName()))
	.." Width: "..tostring(frame:GetWidth())
	.." Height: "..tostring(frame:GetHeight())
	.." Left: "..tostring(frame:GetLeft())
	.." Top: "..tostring(frame:GetTop())
	.." Right: "..tostring(frame:GetRight())
	.." Bottom: "..tostring(frame:GetBottom())
    )

    local anchorPointCount = frame:GetNumPoints()
    if anchorPointCount and anchorPointCount > 0 then 
	for pointIdx = 1, frame:GetNumPoints() do
	    local point, relativeTo, relativePoint, xOffset, yOffset = frame:GetPoint(pointIdx)
	    local relativeToName
	    if relativeTo then
		relativeToName = relativeTo:GetName()
	    end
	    print("point:"..tostring(point)
		.." relativeTo:"..tostring(relativeToName)
		.." relativePoint:"..tostring(relativePoint)
		.." xOffset:"..tostring(xOffset)
		.." yOffset:"..tostring(yOffset) 
	    )
	end
    else
	print("no anchor points: "..tostring(achorPointCount))
    end
end
